@@ -1,6 +1,6 @@
 package Date::Persian::Simple;
 
-$Date::Persian::Simple::VERSION = '0.02';
+$Date::Persian::Simple::VERSION = '0.03';
 
 =head1 NAME
 
@@ -8,13 +8,15 @@ Date::Persian::Simple - Represents Persian date.
 
 =head1 VERSION
 
-Version 0.02
+Version 0.03
 
 =cut
 
 use 5.006;
 use Data::Dumper;
 use Time::localtime;
+use POSIX qw/floor ceil/;
+use Date::Calc qw/Delta_Days/;
 
 use Moo;
 use namespace::clean;
@@ -27,11 +29,31 @@ Represents the Persian date.
 
 =cut
 
+our $PERSIAN_MONTHS = [
+    '',
+    'Farvardin',  'Ordibehesht',  'Khordad',  'Tir',  'Mordad',  'Shahrivar',
+    'Mehr'     ,  'Aban'       ,  'Azar'   ,  'Dey',  'Bahman',  'Esfand'
+];
+
+our $PERSIAN_DAYS = [
+    '<yellow><bold>    Yekshanbeh </bold></yellow>',
+    '<yellow><bold>     Doshanbeh </bold></yellow>',
+    '<yellow><bold>    Seshhanbeh </bold></yellow>',
+    '<yellow><bold> Chaharshanbeh </bold></yellow>',
+    '<yellow><bold>   Panjshanbeh </bold></yellow>',
+    '<yellow><bold>         Jomeh </bold></yellow>',
+    '<yellow><bold>       Shanbeh </bold></yellow>'
+];
+
+has persian_epoch  => (is => 'ro', default => sub { 1948320.5       });
+has persian_days   => (is => 'ro', default => sub { $PERSIAN_DAYS   });
+has persian_months => (is => 'ro', default => sub { $PERSIAN_MONTHS });
+
 has year  => (is => 'rw', predicate => 1);
 has month => (is => 'rw', predicate => 1);
 has day   => (is => 'rw', predicate => 1);
 
-with 'Date::Utils::Persian';
+with 'Date::Utils';
 
 sub BUILD {
     my ($self) = @_;
@@ -45,10 +67,10 @@ sub BUILD {
         my $year  = $today->year + 1900;
         my $month = $today->mon + 1;
         my $day   = $today->mday;
-        my ($y, $m, $d) = $self->gregorian_to_persian($year, $month, $day);
-        $self->year($y);
-        $self->month($m);
-        $self->day($d);
+        my $date  = $self->from_gregorian($year, $month, $day);
+        $self->year($date->year);
+        $self->month($date->month);
+        $self->day($date->day);
     }
 }
 
@@ -60,8 +82,8 @@ sub BUILD {
     # prints today's persian date
     print Date::Persian::Simple->new, "\n";
 
-    my $date = Date::Persian::Simple->new({ year => 1394, month => 1, day => 1 });
-    print "Date: $date\n";
+    # prints given persian date
+    print Date::Persian::Simple->new({ year => 1394, month => 1, day => 1 })->as_string;
 
     # prints equivalent Julian date
     print $date->to_julian, "\n";
@@ -83,20 +105,81 @@ Returns julian date equivalent of the Bahai date.
 sub to_julian {
     my ($self) = @_;
 
-    return $self->persian_to_julian($self->year, $self->month, $self->day);
+    my $epbase = $self->year - (($self->year >= 0) ? 474 : 473);
+    my $epyear = 474 + ($epbase % 2820);
+
+    return $self->day + (($self->month <= 7)?(($self->month - 1) * 31):((($self->month - 1) * 30) + 6)) +
+           floor((($epyear * 682) - 110) / 2816) +
+           ($epyear - 1) * 365 +
+           floor($epbase / 2820) * 1029983 +
+           ($self->persian_epoch - 1);
+}
+
+=head2 from_julian($julian_date)
+
+Returns Persian  date as an object of type L<Date::Persian::Simple> equivalent of
+the Julian date C<$julian_date>.
+
+=cut
+
+sub from_julian {
+    my ($self, $julian_date) = @_;
+
+    $julian_date = floor($julian_date) + 0.5;
+    my $depoch = $julian_date - Date::Persian::Simple->new({ year => 475, month => 1, day => 1 })->to_julian;
+    my $cycle  = floor($depoch / 1029983);
+    my $cyear  = $depoch % 1029983;
+
+    my $ycycle;
+    if ($cyear == 1029982) {
+        $ycycle = 2820;
+    }
+    else {
+        my $aux1 = floor($cyear / 366);
+        my $aux2 = $cyear % 366;
+        $ycycle = floor(((2134 * $aux1) + (2816 * $aux2) + 2815) / 1028522) + $aux1 + 1;
+    }
+
+    my $year = $ycycle + (2820 * $cycle) + 474;
+    if ($year <= 0) {
+        $year--;
+    }
+
+    my $a_persian = Date::Persian::Simple->new({ year => $year, month => 1, day => 1 });
+    my $yday      = ($julian_date - $a_persian->to_julian) + 1;
+    my $month     = ($yday <= 186) ? ceil($yday / 31) : ceil(($yday - 6) / 30);
+    my $b_persian = Date::Persian::Simple->new({ year => $year, month => $month, day => 1 });
+    my $day       = ($julian_date - $b_persian->to_julian) + 1;
+
+    return Date::Persian::Simple->new({ year => $year, month => $month, day => $day });
 }
 
 =head2 to_gregorian()
 
-Returns gregorian date (yyyy-mm-dd) equivalent of the Persian date.
+Returns gregorian date as list (yyyy,mm,dd) equivalent of the Persian date.
 
 =cut
 
 sub to_gregorian {
     my ($self) = @_;
 
-    my @date = $self->julian_to_gregorian($self->to_julian);
-    return sprintf("%04d-%02d-%02d", $date[0], $date[1], $date[2]);
+    return $self->julian_to_gregorian($self->to_julian);
+}
+
+=head2 from_gregorian($year, $month, $day)
+
+Returns Persian  date as an object of type L<Date::Persian::Simple> equivalent of
+the given Gregorian date C<$year>, C<$month> and C<$day>.
+
+=cut
+
+sub from_gregorian {
+    my ($self, $year, $month, $day) = @_;
+
+    $self->validate_date($year, $month, $day);
+    my $julian = $self->gregorian_to_julian($year, $month, $day) +
+                 (floor(0 + 60 * (0 + 60 * 0) + 0.5) / 86400.0);
+    return $self->from_julian($julian);
 }
 
 =head2 day_of_week()
@@ -123,6 +206,51 @@ sub day_of_week {
     return $self->jwday($self->to_julian);
 }
 
+=head2 is_persian_leap_year($year)
+
+Returns 0 or 1 if the given Persian year C<$year> is a leap year or not.
+
+=cut
+
+sub is_persian_leap_year {
+    my ($self, $year) = @_;
+
+    return (((((($year - (($year > 0) ? 474 : 473)) % 2820) + 474) + 38) * 682) % 2816) < 682;
+}
+
+=head2 days_in_persian_month_year($month, $year)
+
+Returns total number of days in the given Persian month year.
+
+=cut
+
+sub days_in_persian_month_year {
+    my ($self, $month, $year) = @_;
+
+    $self->validate_year($year);
+    $self->validate_month($month);
+
+    my @start = Date::Persian::Simple->new({
+        year  => $year,
+        month => $month,
+        day   => 1 })->to_gregorian;
+
+    if ($month == 12) {
+        $year += 1;
+        $month = 1;
+    }
+    else {
+        $month += 1;
+    }
+
+    my @end = Date::Persian::Simple->new({
+        year  => $year,
+        month => $month,
+        day   => 1 })->to_gregorian;
+
+    return Delta_Days(@start, @end);
+}
+
 sub as_string {
     my ($self) = @_;
 
